<?php
/**
 * Dynamically generate CSS code.
 * The code depends on options set in the Highend Options and Post/Page metaboxes.
 *
 * If possible, write the dynamically generated code into a .css file, otherwise return the code. The file is refreshed on each modification of metaboxes & theme options.
 *
 * @package     Sinatra
 * @author      Sinatra Team <hello@sinatrawp.com>
 * @since       1.0.0
 */

/**
 * Do not allow direct script access.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Sinatra_Dynamic_Styles' ) ) :
	/**
	 * Dynamically generate CSS code.
	 */
	class Sinatra_Dynamic_Styles {

		/**
		 * Singleton instance of the class.
		 *
		 * @since 1.0.0
		 * @var object
		 */
		private static $instance;

		/**
		 * URI for Dynamic CSS file.
		 *
		 * @since 1.0.0
		 * @var object
		 */
		private $dynamic_css_uri;

		/**
		 * Path for Dynamic CSS file.
		 *
		 * @since 1.0.0
		 * @var object
		 */
		private $dynamic_css_path;

		/**
		 * Main Sinatra_Dynamic_Styles Instance.
		 *
		 * @since 1.0.0
		 * @return Sinatra_Dynamic_Styles
		 */
		public static function instance() {

			if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Sinatra_Dynamic_Styles ) ) {
				self::$instance = new self();
			}
			return self::$instance;
		}

		/**
		 * Primary class constructor.
		 *
		 * @since 1.0.0
		 */
		public function __construct() {

			$upload_dir = wp_upload_dir();

			$this->dynamic_css_uri  = trailingslashit( $upload_dir['baseurl'] ) . 'sinatra/';
			$this->dynamic_css_path = trailingslashit( $upload_dir['basedir'] ) . 'sinatra/';

			if ( ! is_customize_preview() && wp_is_writable( trailingslashit( $upload_dir['basedir'] ) ) ) {
				add_action( 'sinatra_enqueue_scripts', array( $this, 'enqueue_dynamic_style' ), 20 );
			} else {
				add_action( 'sinatra_enqueue_scripts', array( $this, 'print_dynamic_style' ), 99 );
			}

			// Remove Customizer Custom CSS from wp_head, we will include it in our dynamic file.
			if ( ! is_customize_preview() ) {
				remove_action( 'wp_head', 'wp_custom_css_cb', 101 );
			}

			// Generate new styles on Customizer Save action.
			add_action( 'customize_save_after', array( $this, 'update_dynamic_file' ) );

			// Generate new styles on theme activation.
			add_action( 'after_switch_theme', array( $this, 'update_dynamic_file' ) );

			// Delete the css stye on theme deactivation.
			add_action( 'switch_theme', array( $this, 'delete_dynamic_file' ) );

			// Generate initial dynamic css.
			add_action( 'init', array( $this, 'init' ) );
		}

		/**
		 * Ensure we have dynamic stylesheet generated.
		 *
		 * @since 1.0.0
		 */
		public function init() {

			if ( false === get_transient( 'sinatra_has_dynamic_css' ) ) {
				$this->update_dynamic_file();
			}
		}

		/**
		 * Enqueues dynamic styles file.
		 *
		 * @since 1.0.0
		 */
		public function enqueue_dynamic_style() {

			$exists = file_exists( $this->dynamic_css_path . 'dynamic-styles.css' );

			// Generate the file if it's missing.
			if ( ! $exists ) {
				$exists = $this->update_dynamic_file();
			}

			// Enqueue the file if available.
			if ( $exists ) {
				wp_enqueue_style(
					'sinatra_dynamic_styles',
					$this->dynamic_css_uri . 'dynamic-styles.css',
					false,
					filemtime( $this->dynamic_css_path . 'dynamic-styles.css' ),
					'all'
				);
			}
		}

		/**
		 * Prints inline dynamic styles if writing to file is not possible.
		 *
		 * @since 1.0.0
		 */
		public function print_dynamic_style() {
			$dynamic_css = $this->generate_dynamic_css();
			wp_add_inline_style( 'sinatra-styles', $dynamic_css );
		}

		/**
		 * Generates dynamic CSS code, minifies it and cleans cache.
		 *
		 * @param  boolean $custom_css - should we include the wp_get_custom_css.
		 * @return string, minifed code
		 * @since  1.0.0
		 */
		public function generate_dynamic_css( $custom_css = false ) {

			// Refresh options.
			sinatra()->options->refresh();

			// Delete google fonts enqueue transients.
			delete_transient( 'sinatra_google_fonts_enqueue' );

			// Add our theme custom CSS.
			$css = '';

			// Accent color.
			$accent_color = sinatra_option( 'accent_color' );

			$css .= '
				#si-scroll-top:hover::before,
				.si-btn,
				input[type=submit],
				.comment-form input[type=checkbox]:checked,
				#comments .bypostauthor-badge,
				.single .post-tags a:hover,
				.single .post-category .cat-links a:hover,
				.tagcloud a:hover,
				#main .mejs-controls .mejs-time-rail .mejs-time-current,
				.si-btn.sinatra-read-more::after,
				.post_format-post-format-quote .sinatra-blog-entry-content .quote-post-bg::after,
				.sinatra-hover-slider .post-category a,
				.entry-media > a:hover .entry-media-icon::before,
				.sinatra-pre-footer-cta-style-1 #sinatra-pre-footer .sinatra-flex-row::after,
				.sinatra-pre-footer-cta-style-2 #sinatra-pre-footer::after,
				.select2-container--default .select2-results__option--highlighted[aria-selected],
				.si-input-supported input[type=radio]:checked:before,
				.si-input-supported input[type=checkbox]:checked,
				.sinatra-sidebar-style-2 #secondary .widget-title:before,
				.sinatra-sidebar-style-2 .elementor-widget-sidebar .widget-title:before,
				.widget .cat-item a:hover + span,
				.widget_archive li a:hover + span,
				.widget .cat-item.current-cat a + span,
				#sinatra-footer .widget .cat-item a:hover + span,
				#sinatra-footer .widget_archive li a:hover + span,
				#sinatra-footer .widget .cat-item.current-cat a + span,
				.si-btn.btn-outline:hover,
				.si-hamburger:hover .hamburger-inner,
				.si-hamburger:hover .hamburger-inner::before,
				.si-hamburger:hover .hamburger-inner::after,
				.is-mobile-menu-active .si-hamburger .hamburger-inner,
				.is-mobile-menu-active .si-hamburger .hamburger-inner::before,
				.is-mobile-menu-active .si-hamburger .hamburger-inner::after,
				#infinite-handle span {
					background-color: ' . $accent_color . ';
				}

				.si-btn:hover,
				#infinite-handle span:hover,
				input[type=submit]:hover {
					background-color: ' . sinatra_luminance( $accent_color, .15 ) . ';
				}
				
				mark,
				span.highlight,
				code,
				kbd,
				var,
				samp,
				tt {
					background-color: ' . sinatra_hex2rgba( $accent_color, .09 ) . ';
				}

				code.block {
					background-color: ' . sinatra_hex2rgba( $accent_color, .075 ) . ';
				}

				.content-area a:not(.si-btn),
				#secondary .sinatra-core-custom-list-widget .si-entry a:not(.si-btn),
				#secondary a:not(.si-btn):hover,
				.sinatra-header-widgets .sinatra-header-widget.sinatra-active .si-icon.si-search,
				.sinatra-logo .site-title a:hover,
				#sinatra-header-inner .sinatra-nav > ul > li > a:hover,
				#sinatra-header-inner .sinatra-nav > ul > li.menu-item-has-children:hover > a, 
				#sinatra-header-inner .sinatra-nav > ul > li.current-menu-item > a, 
				#sinatra-header-inner .sinatra-nav > ul > li.current-menu-ancestor > a,
				#sinatra-header-inner .sinatra-nav > ul > li.page_item_has_children:hover > a, 
				#sinatra-header-inner .sinatra-nav > ul > li.current_page_item > a, 
				#sinatra-header-inner .sinatra-nav > ul > li.current_page_ancestor > a,
				#sinatra-topbar .sinatra-nav > ul > li > a:hover,
				#sinatra-topbar .sinatra-nav > ul > li.menu-item-has-children:hover > a, 
				#sinatra-topbar .sinatra-nav > ul > li.current-menu-item > a, 
				#sinatra-topbar .sinatra-nav > ul > li.current-menu-ancestor > a,
				.sinatra-topbar-widget__text a:hover,
				.sinatra-topbar-widget__text a,
				.sinatra-social-nav > ul > li > a .si-icon.bottom-icon,
				.sinatra-header-widgets a:not(.si-btn):hover,
				#sinatra-header-inner .sinatra-header-widgets .sinatra-active,
				.sinatra-pagination .navigation .nav-links .page-numbers:hover,
				.widget .cat-item.current-cat a,
				.widget ul li.current_page_item > a,
				#main .search-form .search-submit:hover,
				#colophon .search-form .search-submit:hover,
				#cancel-comment-reply-link:hover,
				.comment-form .required,
				.navigation .nav-links .page-numbers:hover,
				#main .entry-meta a:hover,
				#main .author-box-title a:hover,
				.single .post-category a,
				.page-links span:hover,
				.site-content .page-links span:hover,
				.navigation .nav-links .page-numbers.current,
				.page-links > span,
				.site-content .page-links > span,
				.si-btn.btn-outline,
				code,
				kbd,
				var,
				samp,
				tt,
				.is-mobile-menu-active .si-hamburger .hamburger-label,
				.si-hamburger:hover .hamburger-label,
				.single #main .post-nav a:hover,
				#sinatra-topbar .sinatra-topbar-widget__text .si-icon {
					color: ' . $accent_color . ';
				}

				#page ::-moz-selection { background-color: ' . $accent_color . '; color: #FFF; }
				#page ::selection { background-color: ' . $accent_color . '; color: #FFF; }

				#comments .comment-actions .reply a:hover,
				.comment-form input[type=checkbox]:checked, 
				.comment-form input[type=checkbox]:focus,
				.comment-form input[type=radio]:checked, 
				.comment-form input[type=radio]:focus,
				.single .post-category a,
				#colophon,
				#secondary .widget-title,
				.elementor-widget-sidebar .widget-title,
				.sinatra-hover-slider .post-category a,
				.si-entry blockquote,
				.wp-block-quote.is-style-large, 
				.wp-block-quote.is-large,
				.navigation .nav-links .page-numbers.current,
				.page-links > span,
				.site-content .page-links > span,
				.si-input-supported input[type=radio]:checked,
				.si-input-supported input[type=checkbox]:checked,
				.si-btn.btn-outline {
					border-color: ' . $accent_color . ';
				}

				#masthead .sinatra-header-widgets .dropdown-item::after,
				.sinatra-nav > ul .sub-menu::after,
				textarea:focus, input[type="text"]:focus, 
				input[type="email"]:focus, 
				input[type=password]:focus, 
				input[type=tel]:focus, 
				input[type=url]:focus, 
				input[type=search]:focus, 
				input[type=date]:focus {
					border-bottom-color: ' . $accent_color . ';
				}

				.sinatra-header-widgets .dropdown-item,
				.preloader-1 > div,
				.sinatra-nav .sub-menu {
					border-top-color: ' . $accent_color . ';
				}

				.sinatra-animate-arrow:hover .arrow-handle,
				.sinatra-animate-arrow:hover .arrow-bar,
				.sinatra-pagination .navigation .nav-links .page-numbers.next:hover .sinatra-animate-arrow .arrow-handle,
				.sinatra-pagination .navigation .nav-links .page-numbers.prev:hover .sinatra-animate-arrow .arrow-handle,
				.sinatra-pagination .navigation .nav-links .page-numbers.next:hover .sinatra-animate-arrow .arrow-bar,
				.sinatra-pagination .navigation .nav-links .page-numbers.prev:hover .sinatra-animate-arrow .arrow-bar {
					fill: ' . $accent_color . ';
				}

				.si-input-supported input[type=checkbox]:focus:hover {
					box-shadow: inset 0 0 0 2px ' . $accent_color . ';
				}
			';

			$header_layout_3_additional_css = '';

			if ( 'layout-3' === sinatra_option( 'header_layout' ) || is_customize_preview() ) {
				$header_layout_3_additional_css = '

					.sinatra-header-layout-3 .sinatra-logo-container > .si-container {
						flex-wrap: wrap;
					}

					.sinatra-header-layout-3 .sinatra-logo-container .sinatra-logo > .logo-inner {
						align-items: flex-start;
					}
					
					.sinatra-header-layout-3 .sinatra-logo-container .sinatra-logo {
						order: 0;
						align-items: flex-start;
						flex-basis: auto;
						margin-left: 0;
					}

					.sinatra-header-layout-3 .sinatra-logo-container .sinatra-header-element {
						flex-basis: auto;
					}

					.sinatra-header-layout-3 .sinatra-logo-container .si-mobile-nav {
						order: 5;
					}

				';
			}

			/**
			 * Top Bar.
			 */

			// Background.
			$css .= $this->print_design_options_field_css( '#sinatra-topbar', 'top_bar_background', 'background' );

			// Border.
			$css .= $this->print_design_options_field_css( '#sinatra-topbar', 'top_bar_border', 'border' );
			$css .= $this->print_design_options_field_css( '.sinatra-topbar-widget', 'top_bar_border', 'separator_color' );

			// Top Bar colors.
			$topbar_color = sinatra_option( 'top_bar_text_color' );

			// Top Bar text color.
			if ( isset( $topbar_color['text-color'] ) && $topbar_color['text-color'] ) {
				$css .= '#sinatra-topbar { color: ' . $topbar_color['text-color'] . '; }';
			}

			// Top Bar link color.
			if ( isset( $topbar_color['link-color'] ) && $topbar_color['link-color'] ) {
				$css .= '
					.sinatra-topbar-widget__text a,
					.sinatra-topbar-widget .sinatra-nav > ul > li > a,
					.sinatra-topbar-widget__socials .sinatra-social-nav > ul > li > a,
					#sinatra-topbar .sinatra-topbar-widget__text .si-icon { 
						color: ' . $topbar_color['link-color'] . '; }
				';
			}

			// Top Bar link hover color.
			if ( isset( $topbar_color['link-hover-color'] ) && $topbar_color['link-hover-color'] ) {
				$css .= '
					#sinatra-topbar .sinatra-nav > ul > li > a:hover,
					#sinatra-topbar .sinatra-nav > ul > li.menu-item-has-children:hover > a,
					#sinatra-topbar .sinatra-nav > ul > li.current-menu-item > a,
					#sinatra-topbar .sinatra-nav > ul > li.current-menu-ancestor > a,
					#sinatra-topbar .sinatra-topbar-widget__text a:hover,
					#sinatra-topbar .sinatra-social-nav > ul > li > a .si-icon.bottom-icon { 
						color: ' . $topbar_color['link-hover-color'] . '; }
				';
			}

			/**
			 * Header.
			 */

			// Background.
			$header_background = sinatra_option( 'header_background' );

			$css .= $this->print_design_options_field_css( '#sinatra-header-inner', 'header_background', 'background' );

			// Font colors.
			$header_color = sinatra_option( 'header_text_color' );

			// Header text color.
			if ( isset( $header_color['text-color'] ) && $header_color['text-color'] ) {
				$css .= '.sinatra-logo .site-description { color: ' . $header_color['text-color'] . '; }';
			}

			// Header link color.
			if ( isset( $header_color['link-color'] ) && $header_color['link-color'] ) {
				$css .= '
					#sinatra-header,
					.sinatra-header-widgets a:not(.si-btn),
					.sinatra-logo a,
					.si-hamburger { 
						color: ' . $header_color['link-color'] . '; }
				';

				$css .= '
					.hamburger-inner,
					.hamburger-inner::before,
					.hamburger-inner::after { 
						background-color: ' . $header_color['link-color'] . ';
					}
				';
			}

			// Header link hover color.
			if ( isset( $header_color['link-hover-color'] ) && $header_color['link-hover-color'] ) {
				$css .= '
					.sinatra-header-widgets a:not(.si-btn):hover, 
					#sinatra-header-inner .sinatra-header-widgets .sinatra-active,
					.sinatra-logo .site-title a:hover, 
					.si-hamburger:hover .hamburger-label, 
					.is-mobile-menu-active .si-hamburger .hamburger-label,
					#sinatra-header-inner .sinatra-nav > ul > li > a:hover,
					#sinatra-header-inner .sinatra-nav > ul > li.menu-item-has-children:hover > a,
					#sinatra-header-inner .sinatra-nav > ul > li.current-menu-item > a,
					#sinatra-header-inner .sinatra-nav > ul > li.current-menu-ancestor > a,
					#sinatra-header-inner .sinatra-nav > ul > li.page_item_has_children:hover > a,
					#sinatra-header-inner .sinatra-nav > ul > li.current_page_item > a,
					#sinatra-header-inner .sinatra-nav > ul > li.current_page_ancestor > a {
						color: ' . $header_color['link-hover-color'] . ';
					}
				';
			}

			// Header border.
			$css .= $this->print_design_options_field_css( '#sinatra-header-inner', 'header_border', 'border' );

			// Header separator color.
			$css .= $this->print_design_options_field_css( '.sinatra-header-widget', 'header_border', 'separator_color' );

			// Main navigation breakpoint.
			$css .= '
				@media screen and (max-width: ' . intval( sinatra_option( 'main_nav_mobile_breakpoint' ) ) . 'px) {
					#sinatra-header-inner .sinatra-nav {
						display: none;
						color: #000;
					}

					.si-mobile-nav {
						display: inline-flex;
					}

					#sinatra-header-inner {
						position: relative;
					}

					#sinatra-header-inner .sinatra-nav > ul > li > a {
						color: inherit;
					}

					#sinatra-header-inner .sinatra-nav-container {
						position: static;
						border: none;
					}

					#sinatra-header-inner .site-navigation {
						display: none;
						position: absolute;
						top: 100%;
						width: 100%;
						left: 0;
						right: 0;
						margin: -1px 0 0;
						background: #FFF;
						border-top: 1px solid #eaeaea;
						box-shadow: 0 15px 25px -10px  rgba(50, 52, 54, 0.125);
						z-index: 999;
						font-size: 1rem;
					}

					#sinatra-header-inner .site-navigation > ul {
						max-height: initial;
						display: block;
					}

					#sinatra-header-inner .site-navigation > ul > li > a {
						padding: 0 !important;
					}

					#sinatra-header-inner .site-navigation > ul li {
						display: block;
						width: 100%;
						padding: 0;
						margin: 0;
						margin-left: 0 !important;
					}

					#sinatra-header-inner .site-navigation > ul .sub-menu {
						position: static;
						display: none;
						border: none;
						box-shadow: none;
						border: 0;
						opacity: 1;
						visibility: visible;
						font-size: rem(14px);
						transform: none;
						background: #f8f8f8;
						pointer-events: all;
						min-width: initial;
						left: 0;
						padding: 0;
						margin: 0;
						border-radius: 0;
						line-height: inherit;
					}

					#sinatra-header-inner .site-navigation > ul .sub-menu > li > a > span {
						padding-left: 35px !important;
					}

					#sinatra-header-inner .site-navigation > ul .sub-menu .sub-menu > li > a > span {
						padding-left: 50px !important;
					}

					#sinatra-header-inner .site-navigation > ul .sub-menu a > span {
						padding: 10px 0 10px 40px;
					}

					#sinatra-header-inner .site-navigation > ul a {
						padding: 0;
						position: relative;
						border-bottom: 1px solid #eaeaea;
						background: none;
					}

					#sinatra-header-inner .site-navigation > ul a > span {
						padding: 10px 20px !important;
						width: 100%;
						display: block;
					}

					#sinatra-header-inner .site-navigation > ul a > span:after,
					#sinatra-header-inner .site-navigation > ul a > span:before {
						display: none !important;
					}

					#sinatra-header-inner .site-navigation > ul a > span.description {
						display: none;
					}

					#sinatra-header-inner .site-navigation > ul .menu-item-has-children > a > span {
						max-width: calc(100% - 50px);
					}

					#sinatra-header-inner .site-navigation > ul .menu-item-has-children > a::after {
						font-family: "sinatra" !important;
						content: "\e92e" !important;
						position: absolute !important;
						top: 0 !important;
						bottom: 0;
						right: 0;
						border-left: 1px solid rgba(0,0,0,.09);
						display: flex;
						align-items: center;
						justify-content: center;
						width: 50px;
						margin: 0 !important;
						font-size: 1em;
						transform: none !important;
					}

					#sinatra-header-inner .site-navigation > ul .menu-item-has-children.si-open > a::after {
						content: "\e931" !important;
					}

					' . $header_layout_3_additional_css . '
				}
			';

			/**
			 * Main Navigation.
			 */

			// Font Color.
			$main_nav_font_color = sinatra_option( 'main_nav_font_color' );

			if ( $main_nav_font_color['link-color'] ) {
				$css .= '#sinatra-header-inner .sinatra-nav > ul > li > a { color: ' . $main_nav_font_color['link-color'] . '; }';
			}

			if ( $main_nav_font_color['link-hover-color'] ) {
				$css .= '
					#sinatra-header-inner .sinatra-nav > ul > li > a:hover,
					#sinatra-header-inner .sinatra-nav > ul > li.menu-item-has-children:hover > a,
					#sinatra-header-inner .sinatra-nav > ul > li.current-menu-item > a,
					#sinatra-header-inner .sinatra-nav > ul > li.current-menu-ancestor > a,
					#sinatra-header-inner .sinatra-nav > ul > li.page_item_has_children:hover > a,
					#sinatra-header-inner .sinatra-nav > ul > li.current_page_item > a,
					#sinatra-header-inner .sinatra-nav > ul > li.current_page_ancestor > a {
						color: ' . $main_nav_font_color['link-hover-color'] . ';
					}
				';
			}

			if ( 'layout-3' === sinatra_option( 'header_layout' ) ) {

				// Background.
				$css .= $this->print_design_options_field_css( '.sinatra-header-layout-3 .sinatra-nav-container', 'main_nav_background', 'background' );

				// Border.
				$css .= $this->print_design_options_field_css( '.sinatra-header-layout-3 .sinatra-nav-container', 'main_nav_border', 'border' );
			}

			// Font size.
			$css .= $this->print_range_field_css( '#sinatra-header-inner', 'font-size', 'main_nav_font_size', false );

			/**
			 * Hero Section.
			 */
			if ( sinatra_option( 'enable_hero' ) ) {
				// Hero height.
				$css .= '#hero .sinatra-hover-slider .hover-slide-item { height: ' . sinatra_option( 'hero_hover_slider_height' ) . 'px; }';
			}

			/**
			 * Pre Footer.
			 */
			if ( sinatra_option( 'enable_pre_footer_cta' ) ) {

				// Call to Action.
				if ( sinatra_option( 'enable_pre_footer_cta' ) ) {

					$cta_style = absint( sinatra_option( 'pre_footer_cta_style' ) );

					// Background.
					$cta_background = sinatra_option( 'pre_footer_cta_background' );

					if ( 1 === $cta_style || is_customize_preview() ) {
						$css .= $this->print_design_options_field_css( '.sinatra-pre-footer-cta-style-1 #sinatra-pre-footer .sinatra-flex-row::after', 'pre_footer_cta_background', 'background' );
					}

					if ( 2 === $cta_style || is_customize_preview() ) {
						$css .= $this->print_design_options_field_css( '.sinatra-pre-footer-cta-style-2 #sinatra-pre-footer::after', 'pre_footer_cta_background', 'background' );
					}

					if ( 'image' === $cta_background['background-type'] && isset( $cta_background['background-color-overlay'] ) && $cta_background['background-color-overlay'] ) {
						$css .= '
							.sinatra-pre-footer-cta-style-1 #sinatra-pre-footer .sinatra-flex-row::before,
			 				.sinatra-pre-footer-cta-style-2 #sinatra-pre-footer::before {
			 					background-color: ' . $cta_background['background-color-overlay'] . ';
			 				}
			 				';
					}

					// Text color.
					$css .= $this->print_design_options_field_css( '#sinatra-pre-footer .h2, #sinatra-pre-footer .h3, #sinatra-pre-footer .h4', 'pre_footer_cta_text_color', 'color' );

					// Border.
					if ( 1 === $cta_style || is_customize_preview() ) {
						$css .= $this->print_design_options_field_css( '.sinatra-pre-footer-cta-style-1 #sinatra-pre-footer .sinatra-flex-row::before', 'pre_footer_cta_border', 'border' );
					}

					if ( 2 === $cta_style || is_customize_preview() ) {
						$css .= $this->print_design_options_field_css( '.sinatra-pre-footer-cta-style-2 #sinatra-pre-footer::before', 'pre_footer_cta_border', 'border' );
					}

					// Font size.
					$css .= $this->print_range_field_css( '#sinatra-pre-footer .h2', 'font-size', 'pre_footer_cta_font_size', true );
				}
			}

			// Footer Background.
			if ( sinatra_option( 'enable_footer' ) || sinatra_option( 'enable_copyright' ) ) {

				// Background.
				$css .= $this->print_design_options_field_css( '#colophon', 'footer_background', 'background' );

				// Footer font color.
				$footer_font_color = sinatra_option( 'footer_text_color' );

				// Footer text color.
				if ( isset( $footer_font_color['text-color'] ) && $footer_font_color['text-color'] ) {
					$css .= '
						#colophon { 
							color: ' . $footer_font_color['text-color'] . ';
						}
					';
				}

				// Footer link color.
				if ( isset( $footer_font_color['link-color'] ) && $footer_font_color['link-color'] ) {
					$css .= '
						#colophon a { 
							color: ' . $footer_font_color['link-color'] . '; 
						}
					';
				}

				// Footer link hover color.
				if ( isset( $footer_font_color['link-hover-color'] ) && $footer_font_color['link-hover-color'] ) {
					$css .= '
						#colophon a:hover,
						#colophon li.current_page_item > a,
						#colophon .sinatra-social-nav > ul > li > a .si-icon.bottom-icon { 
							color: ' . $footer_font_color['link-hover-color'] . ';
						}
					';
				}

				// Footer widget title.
				if ( isset( $footer_font_color['widget-title-color'] ) && $footer_font_color['widget-title-color'] ) {
					$css .= '
						#colophon .widget-title { 
							color: ' . $footer_font_color['widget-title-color'] . ';
						}
					';
				}
			}

			// Main Footer border.
			if ( sinatra_option( 'enable_footer' ) ) {

				// Border.
				$footer_border = sinatra_option( 'footer_border' );

				if ( $footer_border['border-top-width'] ) {
					$css .= '
						#colophon {
							border-top-width: ' . $footer_border['border-top-width'] . 'px;
							border-top-style: ' . $footer_border['border-style'] . ';
							border-top-color: ' . $footer_border['border-color'] . ';
						}
					';
				}

				if ( $footer_border['border-bottom-width'] ) {
					$css .= '
						#sinatra-footer {
							border-bottom-width: ' . $footer_border['border-bottom-width'] . 'px;
							border-bottom-style: ' . $footer_border['border-style'] . ';
							border-bottom-color: ' . $footer_border['border-color'] . ';
						}
					';
				}
			}

			// Sidebar.
			$css .= '
				#secondary {
					width: ' . intval( sinatra_option( 'sidebar_width' ) ) . '%;
				}

				body:not(.sinatra-no-sidebar) #primary {
					max-width: ' . intval( 100 - intval( sinatra_option( 'sidebar_width' ) ) ) . '%;
				}
			';

			// Content background.
			$boxed_content_background_color = sinatra_option( 'boxed_content_background_color' );

			// Boxed Separated Layout specific CSS.
			$css .= '
				.sinatra-layout__boxed-separated.author .author-box, 
				.sinatra-layout__boxed-separated #content, 
				.sinatra-layout__boxed-separated.sinatra-sidebar-style-3 #secondary .si-widget, 
				.sinatra-layout__boxed-separated.sinatra-sidebar-style-3 .elementor-widget-sidebar .si-widget, 
				.sinatra-layout__boxed-separated.blog .sinatra-article, 
				.sinatra-layout__boxed-separated.search-results .sinatra-article, 
				.sinatra-layout__boxed-separated.category .sinatra-article {
					background-color: ' . $boxed_content_background_color . ';
				}

				@media screen and (max-width: 960px) {
					.sinatra-layout__boxed-separated #page {
						background-color: ' . $boxed_content_background_color . ';
					}
				}
			';

			$css .= '
				.sinatra-layout__boxed #page {
					background-color: ' . $boxed_content_background_color . ';
				}
			';

			// Content text color.
			$content_text_color = sinatra_option( 'content_text_color' );

			$css .= '
				body {
					color: ' . $content_text_color . ';
				}

				.comment-form .comment-notes,
				#comments .no-comments,
				#page .wp-caption .wp-caption-text,
				.author-box .author-box-content,
				#comments .comment-meta,
				.comments-closed,
				.entry-meta,
				.si-entry cite,
				legend,
				.sinatra-page-header-description,
				.page-links em,
				.site-content .page-links em,
				.single .entry-footer .last-updated,
				.single .post-nav .post-nav-title,
				#main .widget_recent_comments span,
				#main .widget_recent_entries span,
				#main .widget_calendar table > caption,
				.post-thumb-caption,
				.wp-block-image figcaption,
				.wp-block-embed figcaption {
					color: ' . sinatra_hex2rgba( $content_text_color, 0.73 ) . ';
				}

				.navigation .nav-links .page-numbers svg {
					fill: ' . sinatra_hex2rgba( $content_text_color, 0.73 ) . ';
				}
			';

			// Lightened or darkened background color for backgrounds, borders & inputs.
			$background_color = sinatra_get_background_color();
			
			$content_text_color_offset = sinatra_light_or_dark( $background_color, sinatra_luminance( $background_color, -0.045 ), sinatra_luminance( $background_color, 0.2 ) );

			// Only add for dark background color.
			if ( ! sinatra_is_light_color( $background_color ) ) {
				$css .= '
					#content textarea,
					#content input[type="text"],
					#content input[type="number"],
					#content input[type="email"],
					#content input[type=password],
					#content input[type=tel],
					#content input[type=url],
					#content input[type=search],
					#content input[type=date] {
						background-color: ' . $background_color . ';
					}
				';

				// Offset border color.
				$css .= '
					.sinatra-sidebar-style-3 #secondary .si-widget {
						border-color: ' . $content_text_color_offset . ';
					}
				';

				// Offset background color.
				$css .= '
					.entry-meta .entry-meta-elements > span:before {
						background-color: ' . $content_text_color_offset . ';
					}
				';
			}

			// Content link hover color.
			$css .= '
				.content-area a:not(.si-btn):not(.wp-block-button__link):hover,
				#secondary .sinatra-core-custom-list-widget .si-entry a:not(.si-btn):hover,
				.si-breadcrumbs a:hover {
					color: ' . sinatra_option( 'content_link_hover_color' ) . ';
				}
			';

			// Headings Color.
			$css .= '
				h1, h2, h3, h4, h5, h6,
				.h1, .h2, .h3,
				.sinatra-logo .site-title,
				.error-404 .page-header h1 {
					color: ' . sinatra_option( 'headings_color' ) . ';
				}
			';

			// Container width.
			$css .= '
				.si-container,
				.alignfull > div {
					max-width: ' . sinatra_option( 'container_width' ) . 'px;
				}

				.sinatra-layout__boxed #page {
					max-width: ' . ( intval( sinatra_option( 'container_width' ) ) + 100 ) . 'px;
				}
			';

			// Adjust fullwidth sections for boxed layouts.
			if ( 'boxed' === sinatra_option( 'site_layout' ) || is_customize_preview() ) {
				$css .= '
					@media screen and (max-width: ' . intval( sinatra_option( 'container_width' ) ) . 'px) {
						body.sinatra-layout__boxed.sinatra-no-sidebar .elementor-section.elementor-section-stretched,
						body.sinatra-layout__boxed.sinatra-no-sidebar .si-fw-section,
						body.sinatra-layout__boxed.sinatra-no-sidebar .entry-content .alignfull {
							margin-left: -50px !important;
							margin-right: -50px !important;
						}
					}
				';
			}

			// Logo max width.
			$css .= $this->print_range_field_css( '.sinatra-logo img', 'max-height', 'logo_max_height' );
			$css .= $this->print_range_field_css( '.sinatra-logo img.sinatra-svg-logo', 'height', 'logo_max_height' );

			// Logo margin.
			$css .= $this->print_spacing_field_css( '.sinatra-logo .logo-inner', 'margin', 'logo_margin' );

			/**
			 * Page Header.
			 */
			if ( sinatra_option( 'page_header_enable' ) ) {

				// Page Header.
				$css .= $this->print_range_field_css( '.sinatra-page-header-wrapper', 'min-height', 'page_header_height', true, 'px' );

				// Page Header background.
				$css .= $this->print_design_options_field_css( '.page-header', 'page_header_background', 'background' );

				// Page Header font color.
				$page_header_color = sinatra_option( 'page_header_text_color' );

				// Page Header text color.
				if ( isset( $page_header_color['text-color'] ) && $page_header_color['text-color'] ) {
					$css .= '
						.page-header .page-title { 
							color: ' . $page_header_color['text-color'] . '; }

						.page-header .sinatra-page-header-description,
						.si-single-layout-2 .page-header .entry-meta {
							color: ' . sinatra_hex2rgba( $page_header_color['text-color'], 0.75 ) . '; 
						}

						.si-single-layout-2 .page-header .entry-meta .entry-meta-elements>span:before {
							background-color: ' . sinatra_hex2rgba( $page_header_color['text-color'], 0.5 ) . '; 
						}
					';
				}

				// Page Header link color.
				if ( isset( $page_header_color['link-color'] ) && $page_header_color['link-color'] ) {
					$css .= '
						.page-header .si-breadcrumbs a { 
							color: ' . $page_header_color['link-color'] . '; }

						.page-header .si-breadcrumbs span,
						.page-header .breadcrumb-trail .trail-items li::after, .page-header .si-breadcrumbs .separator {
							color: ' . sinatra_hex2rgba( $page_header_color['link-color'], 0.75 ) . '; 
						}

						.si-single-layout-2.single .page-header .post-category a {
							color: ' . $page_header_color['link-color'] . ';
							border-color: ' . $page_header_color['link-color'] . ';
						}

						.si-single-layout-2.single .page-header .post-category a:hover {
							background-color: ' . $page_header_color['link-color'] . ';
						}
					';
				}

				// Page Header link hover color.
				if ( isset( $page_header_color['link-hover-color'] ) && $page_header_color['link-hover-color'] ) {
					$css .= '
						.page-header .si-breadcrumbs a:hover,
						.si-single-layout-2 .page-header .entry-meta a:hover { 
							color: ' . $page_header_color['link-hover-color'] . '; }
					';
				}

				// Page Header border color.
				$page_header_border = sinatra_option( 'page_header_border' );

				$css .= $this->print_design_options_field_css( '.page-header', 'page_header_border', 'border' );
			}

			/**
			 * Copyright Bar.
			 */
			if ( sinatra_option( 'enable_copyright' ) ) {
				$css .= $this->print_design_options_field_css( '#sinatra-copyright', 'copyright_background', 'background' );

				// Copyright font color.
				$copyright_color = sinatra_option( 'copyright_text_color' );

				// Copyright text color.
				if ( isset( $copyright_color['text-color'] ) && $copyright_color['text-color'] ) {
					$css .= '
						#sinatra-copyright { 
							color: ' . $copyright_color['text-color'] . '; }
					';
				}

				// Copyright link color.
				if ( isset( $copyright_color['link-color'] ) && $copyright_color['link-color'] ) {
					$css .= '
						#sinatra-copyright a { 
							color: ' . $copyright_color['link-color'] . '; }
					';
				}

				// Copyright link hover color.
				if ( isset( $copyright_color['link-hover-color'] ) && $copyright_color['link-hover-color'] ) {
					$css .= '
						#sinatra-copyright a:hover,
						#sinatra-copyright .sinatra-social-nav > ul > li > a .si-icon.bottom-icon,
						#sinatra-copyright .sinatra-nav > ul > li.current-menu-item > a,
						#sinatra-copyright .sinatra-nav > ul > li.current-menu-ancestor > a,
						#sinatra-copyright .sinatra-nav > ul > li:hover > a { 
							color: ' . $copyright_color['link-hover-color'] . '; }
					';
				}
			}

			/**
			 * Typography.
			 */

			// Base HTML font size.
			$css .= $this->print_range_field_css( 'html', 'font-size', 'html_base_font_size', true, 'px' );

			// Font smoothing.
			if ( sinatra_option( 'font_smoothing' ) ) {
				$css .= '
					* {
						-moz-osx-font-smoothing: grayscale;
						-webkit-font-smoothing: antialiased;
					}
				';
			}

			// Body.
			$css .= $this->print_typography_field_css( 'body', 'body_font' );

			// Headings.
			$css .= $this->print_typography_field_css( 'h1, .h1, .sinatra-logo .site-title, .page-header .page-title', 'h1_font' );
			$css .= $this->print_typography_field_css( 'h2, .h2', 'h2_font' );
			$css .= $this->print_typography_field_css( 'h3, .h3', 'h3_font' );
			$css .= $this->print_typography_field_css( 'h4', 'h4_font' );
			$css .= $this->print_typography_field_css( 'h5', 'h5_font' );
			$css .= $this->print_typography_field_css( 'h6', 'h6_font' );
			$css .= $this->print_typography_field_css( 'h1 em, h2 em, h3 em, h4 em, h5 em, h6 em, .h1 em, .h2 em, .h3 em, .sinatra-logo .site-title em, .error-404 .page-header h1 em', 'heading_em_font' );

			// Emphasized Heading.
			$css .= $this->print_typography_field_css( 'h1 em, h2 em, h3 em, h4 em, h5 em, h6 em, .h1 em, .h2 em, .h3 em, .sinatra-logo .site-title em, .error-404 .page-header h1 em', 'heading_em_font' );

			// Site Title font size.
			$css .= $this->print_range_field_css( '#sinatra-header .sinatra-logo .site-title', 'font-size', 'logo_text_font_size', true );

			// Sidebar widget title.
			$css .= $this->print_range_field_css( '#main .widget-title', 'font-size', 'sidebar_widget_title_font_size', true );

			// Footer widget title.
			$css .= $this->print_range_field_css( '#colophon .widget-title', 'font-size', 'footer_widget_title_font_size', true );

			// Page Header - Page Title.
			$css .= $this->print_range_field_css( '#page .page-header .page-title', 'font-size', 'page_header_font_size', true );

			// Blog Single Post - Content Font Size.
			$css .= $this->print_range_field_css( '.single-post .entry-content', 'font-size', 'single_content_font_size', true );

			// Blog Single Post - narrow container.
			if ( 'narrow' === sinatra_option( 'single_content_width' ) ) {
				$css .= '
					.single-post.narrow-content .entry-content > :not([class*="align"]):not([class*="gallery"]):not(.wp-block-image):not(.quote-inner):not(.quote-post-bg), 
					.single-post.narrow-content .mce-content-body:not([class*="page-template-full-width"]) > :not([class*="align"]):not([data-wpview-type*="gallery"]):not(blockquote):not(.mceTemp), 
					.single-post.narrow-content .entry-footer, 
					.single-post.narrow-content .post-nav, 
					.single-post.narrow-content #sinatra-comments-toggle, 
					.single-post.narrow-content #comments, 
					.single-post.narrow-content .entry-content .aligncenter, .single-post.narrow-content .si-narrow-element, 
					.single-post.narrow-content.si-single-layout-1 .entry-header, 
					.single-post.narrow-content.si-single-layout-1 .entry-meta, 
					.single-post.narrow-content.si-single-layout-1 .post-category,
					.single-post.narrow-content.sinatra-no-sidebar .sinatra-page-header-wrapper,
					.single-post.si-single-layout-2.sinatra-no-sidebar.narrow-content .si-breadcrumbs {
						max-width: ' . sinatra_option( 'single_narrow_container_width' ) . 'px;
						margin-left: auto;
						margin-right: auto;
					}

					.single-post.narrow-content .author-box {
						max-width: ' . ( intval( sinatra_option( 'single_narrow_container_width' ) ) + 70 ) . 'px;
					}
				';
			}

			// Allow to be filtered.
			$css = apply_filters( 'sinatra_dynamic_styles', $css );

			// Add user custom CSS.
			if ( $custom_css || ! is_customize_preview() ) {
				$css .= wp_get_custom_css();
			}

			// Minify the CSS code.
			$css = $this->minify_css( $css );

			return $css;
		}

		/**
		 * Update dynamic css file with new CSS. Cleans caches after that.
		 *
		 * @return [Boolean] returns true if successfully updated the dynamic file.
		 */
		public function update_dynamic_file() {

			$css = $this->generate_dynamic_css( true );

			if ( ! $css || '' === trim( $css ) ) {
				return;
			}

			// Load file.php file.
			require_once ABSPATH . 'wp-admin' . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'file.php'; // phpcs:ignore

			global $wp_filesystem;

			// Check if the the global filesystem isn't setup yet.
			if ( is_null( $wp_filesystem ) ) {
				WP_Filesystem();
			}

			$wp_filesystem->mkdir( $this->dynamic_css_path );

			if ( $wp_filesystem->put_contents( $this->dynamic_css_path . 'dynamic-styles.css', $css ) ) {
				$this->clean_cache();
				set_transient( 'sinatra_has_dynamic_css', true, 0 );
				return true;
			}

			return false;
		}

		/**
		 * Delete dynamic css file.
		 *
		 * @return void
		 */
		public function delete_dynamic_file() {

			// Load file.php file.
			require_once ABSPATH . 'wp-admin' . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'file.php'; // phpcs:ignore

			global $wp_filesystem;

			// Check if the the global filesystem isn't setup yet.
			if ( is_null( $wp_filesystem ) ) {
				WP_Filesystem();
			}

			$wp_filesystem->delete( $this->dynamic_css_path . 'dynamic-styles.css' );

			delete_transient( 'sinatra_has_dynamic_css' );
		}

		/**
		 * Simple CSS code minification.
		 *
		 * @param  string $css code to be minified.
		 * @return string, minifed code
		 * @since  1.0.0
		 */
		private function minify_css( $css ) {
			$css = preg_replace( '/\s+/', ' ', $css );
			$css = preg_replace( '/\/\*[^\!](.*?)\*\//', '', $css );
			$css = preg_replace( '/(,|:|;|\{|}) /', '$1', $css );
			$css = preg_replace( '/ (,|;|\{|})/', '$1', $css );
			$css = preg_replace( '/(:| )0\.([0-9]+)(%|em|ex|px|in|cm|mm|pt|pc)/i', '${1}.${2}${3}', $css );
			$css = preg_replace( '/(:| )(\.?)0(%|em|ex|px|in|cm|mm|pt|pc)/i', '${1}0', $css );

			return trim( $css );
		}

		/**
		 * Cleans various caches. Compatible with cache plugins.
		 *
		 * @since 1.0.0
		 */
		private function clean_cache() {

			// If W3 Total Cache is being used, clear the cache.
			if ( function_exists( 'w3tc_pgcache_flush' ) ) {
				w3tc_pgcache_flush();
			}

			// if WP Super Cache is being used, clear the cache.
			if ( function_exists( 'wp_cache_clean_cache' ) ) {
				global $file_prefix;
				wp_cache_clean_cache( $file_prefix );
			}

			// If SG CachePress is installed, reset its caches.
			if ( class_exists( 'SG_CachePress_Supercacher' ) ) {
				if ( method_exists( 'SG_CachePress_Supercacher', 'purge_cache' ) ) {
					SG_CachePress_Supercacher::purge_cache();
				}
			}

			// Clear caches on WPEngine-hosted sites.
			if ( class_exists( 'WpeCommon' ) ) {

				if ( method_exists( 'WpeCommon', 'purge_memcached' ) ) {
					WpeCommon::purge_memcached();
				}

				if ( method_exists( 'WpeCommon', 'clear_maxcdn_cache' ) ) {
					WpeCommon::clear_maxcdn_cache();
				}

				if ( method_exists( 'WpeCommon', 'purge_varnish_cache' ) ) {
					WpeCommon::purge_varnish_cache();
				}
			}

			// Clean OpCache.
			if ( function_exists( 'opcache_reset' ) ) {
				opcache_reset();
			}

			// Clean WordPress cache.
			if ( function_exists( 'wp_cache_flush' ) ) {
				wp_cache_flush();
			}
		}

		/**
		 * Prints spacing field CSS based on passed params.
		 *
		 * @since  1.0.0
		 *
		 * @param  string $css_selector CSS selector.
		 * @param  string $css_property CSS property, such as 'margin', 'padding' or 'border'.
		 * @param  string $setting_id The ID of the customizer setting containing all information about the setting.
		 * @param  bool   $responsive Has responsive values.
		 * @return string  Generated CSS.
		 */
		public function print_spacing_field_css( $css_selector, $css_property, $setting_id, $responsive = true ) {

			// Get the saved setting.
			$setting = sinatra_option( $setting_id );

			// If setting doesn't exist, return.
			if ( ! is_array( $setting ) ) {
				return;
			}

			// Get the unit. Defaults to px.
			$unit = 'px';

			if ( isset( $setting['unit'] ) ) {
				if ( $setting['unit'] ) {
					$unit = $setting['unit'];
				}

				unset( $setting['unit'] );
			}

			// CSS buffer.
			$css_buffer = '';

			// Loop through options.
			foreach ( $setting as $key => $value ) {

				// Check if responsive options are available.
				if ( is_array( $value ) ) {

					if ( 'desktop' === $key ) {
						$mq_open  = '';
						$mq_close = '';
					} elseif ( 'tablet' === $key ) {
						$mq_open  = '@media only screen and (max-width: 768px) {';
						$mq_close = '}';
					} elseif ( 'mobile' === $key ) {
						$mq_open  = '@media only screen and (max-width: 480px) {';
						$mq_close = '}';
					} else {
						$mq_open  = '';
						$mq_close = '';
					}

					// Add media query prefix.
					$css_buffer .= $mq_open . $css_selector . '{';

					// Loop through all choices.
					foreach ( $value as $pos => $val ) {

						if ( empty( $val ) ) {
							continue;
						}

						if ( 'border' === $css_property ) {
							$pos .= '-width';
						}

						$css_buffer .= $css_property . '-' . $pos . ': ' . intval( $val ) . $unit . ';';
					}

					$css_buffer .= '}' . $mq_close;

				} else {

					if ( 'border' === $css_property ) {
						$key .= '-width';
					}

					$css_buffer .= $css_property . '-' . $key . ': ' . intval( $value ) . $unit . ';';
				}
			}

			// Check if field is has responsive values.
			if ( ! $responsive ) {
				$css_buffer = $css_selector . '{' . $css_buffer . '}';
			}

			// Finally, return the generated CSS code.
			return $css_buffer;
		}

		/**
		 * Prints range field CSS based on passed params.
		 *
		 * @since  1.0.0
		 *
		 * @param  string $css_selector CSS selector.
		 * @param  string $css_property CSS property, such as 'margin', 'padding' or 'border'.
		 * @param  string $setting_id The ID of the customizer setting containing all information about the setting.
		 * @param  bool   $responsive Has responsive values.
		 * @param  string $unit Unit.
		 * @return string  Generated CSS.
		 */
		public function print_range_field_css( $css_selector, $css_property, $setting_id, $responsive = true, $unit = 'px' ) {

			// Get the saved setting.
			$setting = sinatra_option( $setting_id );

			// If setting doesn't exist, return.
			if ( ! is_array( $setting ) ) {
				return $css_selector . ' { ' . $css_property . ': ' . $setting . 'px';
			}

			// Get the unit. Defaults to px.
			$unit = 'px';

			if ( isset( $setting['unit'] ) ) {
				if ( $setting['unit'] ) {
					$unit = $setting['unit'];
				}

				unset( $setting['unit'] );
			}

			// CSS buffer.
			$css_buffer = '';

			if ( is_array( $setting ) && ! empty( $setting ) ) {

				// Media query syntax wrap.
				$mq_open  = '';
				$mq_close = '';

				// Loop through options.
				foreach ( $setting as $key => $value ) {

					if ( empty( $value ) ) {
						continue;
					}

					if ( 'desktop' === $key ) {
						$mq_open  = '';
						$mq_close = '';
					} elseif ( 'tablet' === $key ) {
						$mq_open  = '@media only screen and (max-width: 768px) {';
						$mq_close = '}';
					} elseif ( 'mobile' === $key ) {
						$mq_open  = '@media only screen and (max-width: 480px) {';
						$mq_close = '}';
					} else {
						$mq_open  = '';
						$mq_close = '';
					}

					// Add media query prefix.
					$css_buffer .= $mq_open . $css_selector . '{';
					$css_buffer .= $css_property . ': ' . floatval( $value ) . $unit . ';';
					$css_buffer .= '}' . $mq_close;
				}
			}

			// Finally, return the generated CSS code.
			return $css_buffer;
		}

		/**
		 * Prints design options field CSS based on passed params.
		 *
		 * @since 1.0.0
		 * @param string       $css_selector CSS selector.
		 * @param string|mixed $setting The ID of the customizer setting containing all information about the setting.
		 * @param string       $type Design options field type.
		 * @return string      Generated CSS.
		 */
		public function print_design_options_field_css( $css_selector, $setting, $type ) {

			if ( is_string( $setting ) ) {
				// Get the saved setting.
				$setting = sinatra_option( $setting );
			}

			// Setting has to be array.
			if ( ! is_array( $setting ) || empty( $setting ) ) {
				return;
			}

			// CSS buffer.
			$css_buffer = '';

			// Background.
			if ( 'background' === $type ) {

				// Background type.
				$background_type = $setting['background-type'];

				if ( 'color' === $background_type ) {
					if ( isset( $setting['background-color'] ) && ! empty( $setting['background-color'] ) ) {
						$css_buffer .= 'background-color: ' . $setting['background-color'] . ';';
					}
				} elseif ( 'gradient' === $background_type ) {

					$css_buffer .= 'background: ' . $setting['gradient-color-1'] . ';';

					if ( 'linear' === $setting['gradient-type'] ) {
						$css_buffer .= '
							background: -webkit-linear-gradient(' . $setting['gradient-linear-angle'] . 'deg, ' . $setting['gradient-color-1'] . ' ' . $setting['gradient-color-1-location'] . '%, ' . $setting['gradient-color-2'] . ' ' . $setting['gradient-color-2-location'] . '%);
							background: -o-linear-gradient(' . $setting['gradient-linear-angle'] . 'deg, ' . $setting['gradient-color-1'] . ' ' . $setting['gradient-color-1-location'] . '%, ' . $setting['gradient-color-2'] . ' ' . $setting['gradient-color-2-location'] . '%);
							background: linear-gradient(' . $setting['gradient-linear-angle'] . 'deg, ' . $setting['gradient-color-1'] . ' ' . $setting['gradient-color-1-location'] . '%, ' . $setting['gradient-color-2'] . ' ' . $setting['gradient-color-2-location'] . '%);

						';
					} elseif ( 'radial' === $setting['gradient-type'] ) {
						$css_buffer .= '
							background: -webkit-radial-gradient(' . $setting['gradient-position'] . ', circle, ' . $setting['gradient-color-1'] . ' ' . $setting['gradient-color-1-location'] . '%, ' . $setting['gradient-color-2'] . ' ' . $setting['gradient-color-2-location'] . '%);
							background: -o-radial-gradient(' . $setting['gradient-position'] . ', circle, ' . $setting['gradient-color-1'] . ' ' . $setting['gradient-color-1-location'] . '%, ' . $setting['gradient-color-2'] . ' ' . $setting['gradient-color-2-location'] . '%);
							background: radial-gradient(circle at ' . $setting['gradient-position'] . ', ' . $setting['gradient-color-1'] . ' ' . $setting['gradient-color-1-location'] . '%, ' . $setting['gradient-color-2'] . ' ' . $setting['gradient-color-2-location'] . '%);
						';
					}
				} elseif ( 'image' === $background_type ) {
					$css_buffer .= '
						background-image: url(' . $setting['background-image'] . ');
						background-size: ' . $setting['background-size'] . ';
						background-attachment: ' . $setting['background-attachment'] . ';
						background-position: ' . $setting['background-position-x'] . '% ' . $setting['background-position-y'] . '%;
						background-repeat: ' . $setting['background-repeat'] . ';
					';
				}

				$css_buffer = ! empty( $css_buffer ) ? $css_selector . '{' . $css_buffer . '}' : '';

				if ( 'image' === $background_type && isset( $setting['background-color-overlay'] ) && $setting['background-color-overlay'] && isset( $setting['background-image'] ) && $setting['background-image'] ) {
					$css_buffer .= $css_selector . '::after { background-color: ' . $setting['background-color-overlay'] . '; }';
				}
			} elseif ( 'color' === $type ) {

				// Text color.
				if ( isset( $setting['text-color'] ) && ! empty( $setting['text-color'] ) ) {
					$css_buffer .= $css_selector . ' { color: ' . $setting['text-color'] . '; }';
				}

				// Link Color.
				if ( isset( $setting['link-color'] ) && ! empty( $setting['link-color'] ) ) {
					$css_buffer .= $css_selector . ' a { color: ' . $setting['link-color'] . '; }';
				}

				// Link Hover Color.
				if ( isset( $setting['link-hover-color'] ) && ! empty( $setting['link-hover-color'] ) ) {
					$css_buffer .= $css_selector . ' a:hover { color: ' . $setting['link-hover-color'] . ' !important; }';
				}
			} elseif ( 'border' === $type ) {

				// Color.
				if ( isset( $setting['border-color'] ) && ! empty( $setting['border-color'] ) ) {
					$css_buffer .= 'border-color:' . $setting['border-color'] . ';';
				}

				// Style.
				if ( isset( $setting['border-style'] ) && ! empty( $setting['border-style'] ) ) {
					$css_buffer .= 'border-style: ' . $setting['border-style'] . ';';
				}

				// Width.
				$positions = array( 'top', 'right', 'bottom', 'left' );

				foreach ( $positions as $position ) {
					if ( isset( $setting[ 'border-' . $position . '-width' ] ) && ! empty( $setting[ 'border-' . $position . '-width' ] ) ) {
						$css_buffer .= 'border-' . $position . '-width: ' . $setting[ 'border-' . $position . '-width' ] . 'px;';
					}
				}

				$css_buffer = ! empty( $css_buffer ) ? $css_selector . '{' . $css_buffer . '}' : '';

			} elseif ( 'separator_color' === $type && isset( $setting['separator-color'] ) && ! empty( $setting['separator-color'] ) ) {

				// Separator Color.
				$css_buffer .= $css_selector . ':after { background-color:' . $setting['separator-color'] . '; }';
			}

			// Finally, return the generated CSS code.
			return $css_buffer;
		}

		/**
		 * Prints typography field CSS based on passed params.
		 *
		 * @since  1.0.0
		 * @param  string       $css_selector CSS selector.
		 * @param  string|mixed $setting The ID of the customizer setting containing all information about the setting.
		 * @return string       Generated CSS.
		 */
		public function print_typography_field_css( $css_selector, $setting ) {

			if ( is_string( $setting ) ) {
				// Get the saved setting.
				$setting = sinatra_option( $setting );
			}

			// Setting has to be array.
			if ( ! is_array( $setting ) || empty( $setting ) ) {
				return;
			}

			// CSS buffer.
			$css_buffer = '';

			// Properties.
			$properties = array(
				'font-weight',
				'font-style',
				'text-transform',
				'text-decoration',
			);

			foreach ( $properties as $property ) {

				if ( 'inherit' !== $setting[ $property ] ) {
					$css_buffer .= $property . ':' . $setting[ $property ] . ';';
				}
			}

			$font_family = sinatra()->fonts->get_font_family( $setting['font-family'] );

			$css_buffer .= 'font-family: ' . $font_family . ';';

			// Letter spacing.
			if ( ! empty( $setting['letter-spacing'] ) ) {
				$css_buffer .= 'letter-spacing:' . $setting['letter-spacing'] . $setting['letter-spacing-unit'] . ';';
			}

			// Font size.
			if ( ! empty( $setting['font-size-desktop'] ) ) {
				$css_buffer .= 'font-size:' . $setting['font-size-desktop'] . $setting['font-size-unit'] . ';';
			}

			// Line Height.
			if ( ! empty( $setting['line-height-desktop'] ) ) {
				$css_buffer .= 'line-height:' . $setting['line-height-desktop'] . ';';
			}

			$css_buffer = $css_buffer ? $css_selector . '{' . $css_buffer . '}' : '';

			// Responsive options - tablet.
			$tablet = '';
			
			if ( ! empty( $setting['font-size-tablet'] ) ) {
				$tablet .= 'font-size:' . $setting['font-size-tablet'] . $setting['font-size-unit'] . ';';
			}

			if ( ! empty( $setting['line-height-tablet'] ) ) {
				$tablet .= 'line-height:' . $setting['line-height-tablet'] . ';';
			}

			$tablet = ! empty( $tablet ) ? '@media only screen and (max-width: 768px) {' . $css_selector . '{' . $tablet . '} }' : '';

			$css_buffer .= $tablet;

			// Responsive options - mobile.
			$mobile = '';

			if ( ! empty( $setting['font-size-mobile'] ) ) {
				$mobile .= 'font-size:' . $setting['font-size-mobile'] . $setting['font-size-unit'] . ';';
			}

			if ( ! empty( $setting['line-height-mobile'] ) ) {
				$mobile .= 'line-height:' . $setting['line-height-mobile'] . ';';
			}

			$mobile = ! empty( $mobile ) ? '@media only screen and (max-width: 480px) {' . $css_selector . '{' . $mobile . '} }' : '';

			$css_buffer .= $mobile;

			// Equeue google fonts.
			if ( sinatra()->fonts->is_google_font( $setting['font-family'] ) ) {

				$params = array();

				if ( 'inherit' !== $setting['font-weight'] ) {
					$params['weight'] = $setting['font-weight'];
				}

				if ( 'inherit' !== $setting['font-style'] ) {
					$params['style'] = $setting['font-style'];
				}

				if ( $setting['font-subsets'] && ! empty( $setting['font-subsets'] ) ) {
					$params['subsets'] = $setting['font-subsets'];
				}

				sinatra()->fonts->enqueue_google_font(
					$setting['font-family'],
					$params
				);
			}

			// Finally, return the generated CSS code.
			return $css_buffer;
		}
	}
endif;

/**
 * The function which returns the one Sinatra_Dynamic_Styles instance.
 *
 * Use this function like you would a global variable, except without needing
 * to declare the global.
 *
 * Example: <?php $dynamic_styles = sinatra_dynamic_styles(); ?>
 *
 * @since 1.0.0
 * @return object
 */
function sinatra_dynamic_styles() {
	return Sinatra_Dynamic_Styles::instance();
}

sinatra_dynamic_styles();
